{ use 5.006; }
use warnings;
use strict;

use Module::Build;

Module::Build->new(
	module_name => "XML::Easy::SimpleSchemaUtil",
	license => "perl",
	configure_requires => {
		"Module::Build" => 0,
		"perl" => "5.006",
		"strict" => 0,
		"warnings" => 0,
	},
	build_requires => {
		"Module::Build" => 0,
		"Test::More" => 0,
		"XML::Easy::NodeBasics" => "0.007",
		"perl" => "5.006",
		"strict" => 0,
		"warnings" => 0,
	},
	requires => {
		"Params::Classify" => 0,
		"XML::Easy::Classify" => "0.006",
		"XML::Easy::NodeBasics" => "0.007",
		"XML::Easy::Syntax" => 0,
		"parent" => 0,
		"perl" => "5.006",
		"strict" => 0,
		"warnings" => 0,
	},
	dynamic_config => 0,
	meta_add => { distribution_type => "module" },
	meta_merge => {
		"meta-spec" => { version => "2" },
		resources => {
			bugtracker => {
				mailto => "bug-XML-Easy-SimpleSchemaUtil".
					"\@rt.cpan.org",
				web => "https://rt.cpan.org/Public/Dist/".
					"Display.html?Name=".
					"XML-Easy-SimpleSchemaUtil",
			},
		},
	},
	sign => 1,
)->create_build_script;

1;
